/* -*- coding: utf-8 -*-
 * :Project:   hurm -- Application stylesheet
 * :Created:   mar 02 feb 2016 12:43:02 CET
 * :Author:    Lele Gaifax <lele@metapensiero.it>
 * :License:   GNU General Public License version 3 or later
 * :Copyright: © 2016 Lele Gaifax
 */

/* Generic */

.mp-desktop-start-button-icon {
    background-image: url(../favicon.png);
}

.x-btn-icon-el.edit-record-icon,
.x-menu-item-icon.edit-record-icon {
    background-image: url(../images/application_form_edit.png);
}

.x-btn-icon-el.print-icon,
.x-menu-item-icon.print-icon {
    background-image: url('../images/modules/printer.png');
}

.x-btn-icon-el.pdf-icon,
.x-menu-item-icon.pdf-icon {
    background-image: url('../images/modules/pdf.png');
}


/* Activities */

.x-btn-icon-el.activities-icon,
.x-menu-item-icon.activities-icon,
.x-window-header-icon.activities-icon {
    background-image: url('../images/modules/activities/tag_blue.png');
}

.x-window-header-icon.edit-activity-icon {
    background-image: url(../images/modules/activities/tag_blue_edit.png);
}


/* Duties */

.x-btn-icon-el.duties-icon,
.x-window-header-icon.duties-icon {
    background-image: url('../images/modules/duties/user_time.png');
}


/* Editions */

.x-btn-icon-el.editions-icon,
.x-menu-item-icon.editions-icon,
.x-window-header-icon.editions-icon {
    background-image: url('../images/modules/editions/date16.png');
}

.mp-desktop-shortcut-icon.editions-icon {
    background-image: url('../images/modules/editions/date48.png');
}

.x-window-header-icon.edit-edition-icon {
    background-image: url(../images/modules/editions/calendar_edit.png);
}

.x-btn-icon-el.download-icon,
.x-menu-item-icon.download-icon {
    background-image: url('../images/modules/editions/application_put.png');
}

.x-btn-icon-el.yaml-icon,
.x-menu-item-icon.yaml-icon {
    background-image: url('../images/modules/editions/page_white_put.png');
}

.x-btn-icon-el.excel-icon,
.x-menu-item-icon.excel-icon {
    background-image: url('../images/modules/editions/page_white_excel.png');
}

/* Locations */

.x-btn-icon-el.locations-icon,
.x-menu-item-icon.locations-icon,
.x-window-header-icon.locations-icon {
    background-image: url('../images/modules/locations/houses16.png');
}

.mp-desktop-shortcut-icon.locations-icon {
    background-image: url('../images/modules/locations/houses48.png');
}

.x-window-header-icon.edit-location-icon {
    background-image: url(../images/modules/locations/home.png);
}


/* Persons */

.x-btn-icon-el.persons-icon,
.x-menu-item-icon.persons-icon,
.x-window-header-icon.persons-icon {
    background-image: url('../images/modules/persons/users16.png');
}

.mp-desktop-shortcut-icon.persons-icon {
    background-image: url('../images/modules/persons/users48.png');
}

.x-window-header-icon.edit-person-icon {
    background-image: url(../images/modules/persons/user_edit.png);
}


/* Tasks */

.x-btn-icon-el.tasks-icon,
.x-window-header-icon.tasks-icon {
    background-image: url('../images/modules/tasks/list_bullet.png');
}
